using System;
using System.Collections;
using System.Collections.Generic;
using InABox.Core;

namespace InABox.Mobile
{

    public interface IListModel : IModel
    {
        object CreateItem();
        void CommitItem(object item);
        object AddItem();
        void DeleteItem(object item);
        IEnumerable Items { get; }
        void Search();

        void ToggleSelection(object item);
        void SelectItem(object item);
        void UnselectItem(object item);
        void SelectAll();
        void SelectNone();
        bool IsSelected(object item);
        void SetSelectedItems(IEnumerable<object> items);
        IEnumerable SelectedItems { get; }

    }

    public class ListModelItemCreatedArgs<TShell> : EventArgs
    {
        public TShell Item { get; private set; }

        public ListModelItemCreatedArgs(TShell item)
        {
            Item = item;
        }
    }

    public delegate void ListModelItemCreatedEvent<TShell>(object sender, ListModelItemCreatedArgs<TShell> args);
    
    public interface IListModel<TModel, TShell, TEntity> : IListModel
        where TModel : IListModel
        where TEntity : Entity, IRemotable, IPersistent, new() 
        where TShell : Shell<TModel,TEntity>, new()
    {
        //IList<TShell> Items { get; set; }
        event ListModelItemCreatedEvent<TShell> ItemAdded;
    }
}